package example.localplugin;

import gridsim.dcworms.DCWormsTags;

import java.util.List;
import java.util.Random;

import schedframe.events.scheduling.SchedulingEvent;
import schedframe.resources.computing.ComputingNode;
import schedframe.resources.computing.profiles.energy.airthroughput.StandardAirThroughputStateName;
import schedframe.scheduling.manager.resources.ClusterResourceManager;
import schedframe.scheduling.manager.resources.ResourceManager;
import schedframe.scheduling.manager.tasks.JobRegistry;
import schedframe.scheduling.plan.SchedulingPlanInterface;
import schedframe.scheduling.plan.impl.SchedulingPlan;
import schedframe.scheduling.plugin.grid.ModuleList;
import schedframe.scheduling.queue.TaskQueue;
import schedframe.scheduling.queue.TaskQueueList;
import schedframe.scheduling.tasks.TaskInterface;

public class FCFSBF_FanManagementClusterPlugin extends BaseLocalSchedulingPlugin {

	private Random rand;
	
	public FCFSBF_FanManagementClusterPlugin () {
		rand = new Random(5);
	}

	public SchedulingPlanInterface<?> schedule(SchedulingEvent event, TaskQueueList queues, JobRegistry jobRegistry,
			 ResourceManager resManager, ModuleList modules) {

		ClusterResourceManager resourceManager = (ClusterResourceManager) resManager;
		SchedulingPlan plan = new SchedulingPlan();
		// choose the events types to serve.
		// Different actions for different events are possible.
		switch (event.getType()) {
		case START_TASK_EXECUTION:
		case TASK_FINISHED:
		//case TIMER:
			// our tasks are placed only in first queue (see
			// BaseLocalSchedulingPlugin.placeJobsInQueues() method)
			TaskQueue q = queues.get(0);
			List<ComputingNode> notSelectedNodes = resourceManager.getComputingNodes();
			// check all tasks in queue
			for (int i = 0; i < q.size(); i++) {
				TaskInterface<?> task = q.get(i);
				// if status of the tasks in READY
				if (task.getStatus() == DCWormsTags.READY) {

					ComputingNode node = chooseRandomProvider(resourceManager);
					if (node != null) {
						node.getAirThroughputInterface().setAirThroughputState(StandardAirThroughputStateName.FAN_ON);
						notSelectedNodes.remove(node);
						addToSchedulingPlan(plan, task, node.getName());
					}
				}
			}
			turnOffIdleNodes(notSelectedNodes);
			break;
		}
		return plan;
	}

	private ComputingNode chooseRandomProvider(ClusterResourceManager resourceManager) {
		List<ComputingNode> nodes = resourceManager.getComputingNodes();
		int nodeIdx = rand.nextInt(nodes.size());
		return nodes.get(nodeIdx);
	}
	
	private void turnOffIdleNodes(List<ComputingNode> nodes){
		for(ComputingNode node : nodes){
			if(node.getFreeProcessorsNumber() == node.getProcessorsNumber()){
				node.getAirThroughputInterface().setAirThroughputState(StandardAirThroughputStateName.FAN_OFF);
			}
		}
	}


}